unit InstallReader;

interface

uses
	Windows, Classes, Registry, DsgnIntf;

type
	TRORegistry = class(TRegistry)
	public
		function OpenKeyRO(const Key: string; CanCreate: Boolean): Boolean;
	end;

	TInstallReader = class(TComponent)
	private
		FReg: TRORegistry;
		FAuthor,
		FUserName,
		FCompany,
		FVersionPath,
		FActualVersion,
		FDefaultVersion,
		FErrorReading,
		FLocalMachinePath,
		FVersionValue: string;
	protected
		procedure SetLMPath(Value: string);
		procedure SetDVersion(Value: string);
		procedure SetVPath(Value: string);
		procedure SetVValue(Value: string);
		function	GetVersion: string;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure Loaded; override;
		procedure ReadInstall; virtual;
	published
		property ActualVersion: string
			read 	FActualVersion
			write FActualVersion;
		property Author: string
			read	FAuthor
			write	FAuthor;
		property Company: string
			read 	FCompany
			write FCompany;
		property DefaultVersion: string
			read 	FDefaultVersion
			write SetDVersion;
		property ErrorReading: string
			read	FErrorReading
			write FErrorReading;
		property LocalMachinePath: string
			read 	FLocalMachinePath
			write SetLMPath;
		property UserName: string
			read 	FUserName
			write FUserName;
		property VersionPath: string
			read 	FVersionPath
			write SetVPath;
		property VersionValue: string
			read FVersionValue
			write SetVValue;
	end; // TInstallReader

	TStringROProperty = class(TStringProperty)
	public
		function GetAttributes: TPropertyAttributes; override;
		function GetName: string; override;
	end; // TStringROProperty

	TAuthorProperty = class(TStringProperty)
	public
		function GetAttributes: TPropertyAttributes; override;
		function GetName: string; override;
		procedure Edit; override;
	end; // TStringROProperty

procedure Register;

resourcestring sNotReadable = '(Not Readable)';

implementation

// TRORegistry Implementation

function IsRelative(const Value: string): Boolean;
begin
	Result := not ((Value <> '') and (Value[1] = '\'));
end;

function TRORegistry.OpenKeyRO(const Key: string; CanCreate: Boolean): Boolean;
var
	TempKey: HKey;
	S: string;
	Disposition: Integer;
	Relative: Boolean;
begin
	S := Key;
	Relative := IsRelative(S);
	if not Relative then Delete(S, 1, 1);
	TempKey := 0;
	if not CanCreate or (S = '') then
	begin
		Result := RegOpenKeyEx(GetBaseKey(Relative), PChar(S), 0,
			KEY_ALL_ACCESS, TempKey) = ERROR_SUCCESS;
  end else
    Result := RegCreateKeyEx(GetBaseKey(Relative), PChar(S), 0, nil,
      REG_OPTION_NON_VOLATILE, KEY_READ, nil, TempKey, @Disposition) = ERROR_SUCCESS;
	if Result then
	begin
		if (CurrentKey <> 0) and Relative then S := CurrentPath + '\' + S;
		ChangeKey(TempKey, S);
	end;
end; // OpenKeyRO

// TInstallReader Implementation

constructor TInstallReader.Create(AOwner: TComponent);
begin
	inherited;
	FReg := TRORegistry.Create;
	FActualVersion := sNotReadable;
	FAuthor := ' 1998 Christophe Porteneuve a.k.a. TDD';
	FCompany := sNotReadable;
	FDefaultVersion := '1.0';
	FErrorReading := '(No)';
	FLocalMachinePath := 'Software';
	FUserName := sNotReadable;
	FVersionPath := 'Software';
	FVersionValue := 'Version';
end; // Create

destructor TInstallReader.Destroy;
begin
	FReg.Free;
	inherited;
end; // Destroy

procedure TInstallReader.Loaded;
begin
	inherited;
	ReadInstall;
end; // Loaded

procedure TInstallReader.SetLMPath(Value: string);
begin
	if Value = FLocalMachinePath then
		Exit;
	FLocalMachinePath := Value;
	if FLocalMachinePath[Length(FLocalMachinePath)] = '\' then
		System.Delete(FLocalMachinePath, Length(FLocalMachinePath), 1);
	if
		(FLocalMachinePath <> '') and (FVersionPath <> '') and
		(FVersionValue <> '') then
		ReadInstall;
end; // SetLMPath

procedure TInstallReader.SetVPath(Value: string);
begin
	if Value = FVersionPath then
		Exit;
	FVersionPath := Value;
	if FVersionPath[Length(FVersionPath)] = '\' then
		System.Delete(FVersionPath, Length(FVersionPath), 1);
	if
		(FVersionPath <> '') and (FLocalMachinePath <> '') and
		(FVersionValue <> '') then
		ReadInstall;
end; // SetVPath

procedure TInstallReader.SetDVersion(Value: string);
begin
	if Value = FDefaultVersion then
		Exit;
	FDefaultVersion := Value;
	if
		(FVersionPath <> '') and (FLocalMachinePath <> '') and
		(FVersionValue <> '') then
		ReadInstall;
end; // SetDVersion

procedure TInstallReader.SetVValue(Value: string);
begin
	if Value = FVersionValue then
		Exit;
	FVersionValue := Value;
	if (Value <> '') and (FVersionPath <> '') and (FLocalMachinePath <> '') then
		ReadInstall;
end; // SetVValue

function TInstallReader.GetVersion: string;
begin
	if FActualVersion <> sNotReadable then
		Result := FActualVersion
	else
		Result := FDefaultVersion;
end; // GetVersion

procedure TInstallReader.ReadInstall;
begin
	if [csLoading, csReading] * ComponentState <> [] then
		Exit;
	FActualVersion := sNotReadable;
	FUserName := sNotReadable;
	FCompany := sNotReadable;
	FErrorReading := '(Yes)';
	try
		FReg.RootKey := HKEY_CURRENT_USER;
		if FReg.OpenKeyRO(FVersionPath, False) then
			FActualVersion := FReg.ReadString(FVersionValue);
		if FActualVersion = '' then
			FActualVersion := sNotReadable;
		FReg.RootKey := HKEY_LOCAL_MACHINE;
		if FReg.OpenKeyRO(FLocalMachinePath+'\'+GetVersion, False) then
		begin
			UserName 	:= FReg.ReadString('Name');
			if UserName = '' then
				UserName := sNotReadable;
			Company		:= FReg.ReadString('Company');
			if Company = '' then
				Company := sNotReadable;
		end;
		if (UserName <> sNotReadable) and (Company <> sNotReadable) then
			FErrorReading := '(No)';
	except
	end;
end; // ReadInstall

function TStringROProperty.GetAttributes: TPropertyAttributes;
begin
	Result := inherited GetAttributes + [paReadOnly];
end; // GetAttributes

function TStringROProperty.GetName: string;
begin
	Result := inherited GetName + '*';
end; // GetName

function TAuthorProperty.GetAttributes: TPropertyAttributes;
begin
	Result := inherited GetAttributes + [paReadOnly, paDialog];
end; // GetAttributes

function TAuthorProperty.GetName: string;
begin
	Result := 'Who wrote me?*';
end; // GetName

procedure TAuthorProperty.Edit;
begin
	MessageBox(0,
		'TInstallReader v.1.0'#13#10+
		' 1998 Christophe Porteneuve a.k.a. TDD'#13#10+
		'Email: porten_c@epita.fr'#13#10+
		'Web: http://www.epita.fr/~porten_c',
		'About TInstallReader...',
		MB_ICONINFORMATION+MB_OK);
end; // Edit

procedure Register;
begin
	RegisterComponents('TDDians', [TInstallReader]);
	RegisterPropertyEditor(TypeInfo(string), TInstallReader, 'ActualVersion',
		TStringROProperty);
	RegisterPropertyEditor(TypeInfo(string), TInstallReader, 'Company',
		TStringROProperty);
	RegisterPropertyEditor(TypeInfo(string), TInstallReader, 'ErrorReading',
		TStringROProperty);
	RegisterPropertyEditor(TypeInfo(string), TInstallReader, 'UserName',
		TStringROProperty);
	RegisterPropertyEditor(TypeInfo(string), TInstallReader, 'Author',
		TAuthorProperty);
end;

end.
